==> InfiniTec.Exchange_0.96_Source.zip: WebDav/AttachmentCollection.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;
using InfiniTec.Threading;
using InfiniTec.Exchange.WebDav;
using System.ComponentModel;
using System.Collections;
using InfiniTec.Exchange.WebDav.Request;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class manages access to attachments of an Exchange item.
    /// </summary>
    public class AttachmentCollection : ObjectBase, IEnumerable, IEnumerable<Attachment>
    {
        /// <summary>
        /// This event is fired when the status of the <see cref="Add" /> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> AddProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Add"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler AddCompleted;
        /// <summary>
        /// This event fires, when the <see cref="Refresh"/> operation finishes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> RefreshProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Refresh"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler RefreshCompleted;
        /// <summary>
        /// This event is fired when the status of the <see cref="Remove" /> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> RemoveProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Remove"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler RemoveCompleted;

        List<Attachment> _Attachments;

        internal AttachmentCollection(Uri url, Connection connection)
            : base(url, connection)
        {
            _Attachments = new List<Attachment>();
        }

        /// <summary>
        /// Returns the number of attachments on the current item.
        /// </summary>
        public int Count
        {
            get { return _Attachments.Count; }
        }

        /// <summary>
        /// Returns the <see cref="Attachment"/> with the specified index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Attachment this[int index]
        {
            get
            {
                return _Attachments[index];
            }
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to add an attachment to an item.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content for the attachment.</param>
        /// <returns></returns>
        public AsyncOperation<RequestStatus> GetAddOperation(string name, Stream stream)
        {
            Uri url;

            url = new Uri(Url, name);

            return new AsyncOperation<RequestStatus>(AddInternal(name, stream));
        }


        private IEnumerable<OperationAction> AddInternal(string name, Stream stream)
        {
            int index;
            Uri url;

            url = new Uri(Url.AbsoluteUri + "/" + name);

            using (AsyncOperation<RequestStatus> operation = Methods.Put(url, stream))
            {
                yield return new NestedOperation(operation, true);
            }

            index = -1;
            for (int i = 0; i < _Attachments.Count; i++)
            {
                if (!_Attachments[i].Name.Equals(name, StringComparison.CurrentCultureIgnoreCase)) continue;

                index = i;
                break;
            }
            if (index != -1) _Attachments[index] = new Attachment(Url, name, 0, Connection);
            else _Attachments.Add(new Attachment(Url, name, stream.CanSeek ? (int) stream.Length : 0, Connection));
        }

        /// <summary>
        /// Adds an attachment to the current item using the specified name and stream. The current thread is blocked until
        /// the operation completes.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content of the attachment.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Add(string name, Stream stream)
        {
            using (AsyncOperation<RequestStatus> operation = GetAddOperation(name, stream))
            {
                operation.Run();

                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Asynchronously adds an attachment to the current item using the specified name and stream.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content of the attachment.</param>
        public void AddAsync(string name, Stream stream)
        {
            AsyncOperation<RequestStatus> operation;

            InitiatePendingAsyncOperation();

            operation = GetAddOperation(name, stream);

            operation.ProgressChanged += AddProgressChanged;
            operation.Completed += Add_Completed;
            operation.Error += Add_Error;

            RegisterPendingAsyncOperation(operation);
            operation.Run();
        }

        void Add_Error(object sender, ExceptionEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnAddCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Add_Completed(object sender, CompletedEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnAddCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="AddCompleted"/> event.
        /// </summary>
        /// <param name="e">The parameters of the event.</param>
        protected virtual void OnAddCompleted(AsyncCompletedEventArgs e)
        {
            if (AddCompleted != null) AddCompleted(this, e);
        }

        /// <summary>
        /// Returns a <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to refresh this instance.
        /// </summary>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetRefreshOperation()
        {
            return new AsyncOperation<RequestStatus>(RefreshInternal());
        }

        private IEnumerable<OperationAction> RefreshInternal()
        {
            using (AsyncOperation<RequestStatus, PropertyContainer[]> operation = Methods.X_MS_EnumAtts(Url))
            {
                yield return new NestedOperation(operation, true);

                foreach (PropertyContainer container in operation.Result)
                {
                    _Attachments.Add(new Attachment(container, Connection));
                }
            }
        }

        /// <summary>
        /// Reads the attachment information from the current item. The current thread is blocked while the request is in progress.
        /// </summary>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Refresh()
        {
            using (AsyncOperation<RequestStatus> operation = GetRefreshOperation())
            {
                operation.Run();
                operation.WaitForCompletion();
            }
        }
        /// <summary>
        /// Asynchronously reads the attachment information from the current item. 
        /// </summary>
        public void RefreshAsync()
        {
            AsyncOperation<RequestStatus> operation;

            InitiatePendingAsyncOperation();

            operation = GetRefreshOperation();
            RegisterPendingAsyncOperation(operation);
            operation.ProgressChanged += RefreshProgressChanged;
            operation.Completed += Refresh_Completed;
            operation.Error += Refresh_Error;

            operation.Run();
        }

        void Refresh_Error(object sender, ExceptionEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnRefreshCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Refresh_Completed(object sender, CompletedEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnRefreshCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="RefreshCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters of the event.</param>
        protected virtual void OnRefreshCompleted(AsyncCompletedEventArgs e)
        {
            if (RefreshCompleted != null) RefreshCompleted(this, e);
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to remoave the specified attachment.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetRemoveOperation(Attachment attachment)
        {
            return new AsyncOperation<RequestStatus>(RemoveInternal(attachment));
        }

        private IEnumerable<OperationAction> RemoveInternal(Attachment attachment)
        {
            if (!_Attachments.Contains(attachment)) throw new InvalidOperationException("The specified attachment is not an element of this AttachmentCollection instance");

            using (AsyncOperation<RequestStatus> operation = Methods.Delete(attachment.Url))
            {
                yield return new NestedOperation(operation, true);
            }
            _Attachments.Remove(attachment);
        }

        /// <summary>
        /// Removes the specified <see cref="Attachment"/> from the item. The current thread is blocked until the operation 
        /// completes.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Remove(Attachment attachment)
        {
            using (AsyncOperation<RequestStatus> operation = GetRemoveOperation(attachment))
            {
                operation.Run();
                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Removes the <see cref="Attachment"/> with the specified index. The current thread is blocked until the operation 
        /// completes.
        /// </summary>
        /// <param name="index">The index of the attachment to remove.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void RemoveAt(int index)
        {
            Remove(this[index]);
        }
        
        /// <summary>
        /// Asynchronously removes the <see cref="Attachment"/> with the specified index from the item.
        /// </summary>
        /// <param name="index">The index of the <see cref="Attachment"/> to remove.</param>
        public void RemoveAtAsync(int index)
        {
            RemoveAsync(this[index]);
        }
        /// <summary>
        /// Asynchronously removes the specified <see cref="Attachment"/> from the item.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        public void RemoveAsync(Attachment attachment)
        {
            AsyncOperation<RequestStatus> operation;

            InitiatePendingAsyncOperation();

            operation = GetRemoveOperation(attachment);

            operation.ProgressChanged += RemoveProgressChanged;
            operation.Completed += Remove_Completed;
            operation.Error += Remove_Error;

            RegisterPendingAsyncOperation(operation);
            operation.Run();
        }

        void Remove_Error(object sender, ExceptionEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnRemoveCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Remove_Completed(object sender, CompletedEventArgs e)
        {
            UnregisterPendingAsyncOperation();

            OnRemoveCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="RemoveCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters.</param>
        protected virtual void OnRemoveCompleted(AsyncCompletedEventArgs e)
        {
            if (RemoveCompleted != null) RemoveCompleted(this, e);
        }

        #region IEnumerable<Attachment> Members

        /// <summary>
        /// Returns an <see cref="IEnumerator"/> which can be used to iterate through the collection of attachemnts.
        /// </summary>
        /// <returns>An <see cref="IEnumerator"/> instance.</returns>
        public IEnumerator<Attachment> GetEnumerator()
        {
            return _Attachments.GetEnumerator();
        }

        #endregion


        #region IEnumerable Members

        /// <summary>
        /// Returns an <see cref="IEnumerator"/> which can be used to iterate through the collection of attachemnts.
        /// </summary>
        /// <returns>An <see cref="IEnumerator"/> instance.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable)_Attachments).GetEnumerator();
        }

        #endregion
    }
}

==> InfiniTec.Exchange_0.97_Source.zip: WebDav/AttachmentCollection.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;
using InfiniTec.Threading;
using InfiniTec.Exchange.WebDav;
using System.ComponentModel;
using System.Collections;
using InfiniTec.Exchange.WebDav.Request;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class manages access to attachments of an Exchange item.
    /// </summary>
    public class AttachmentCollection : ObjectBase, IEnumerable, IEnumerable<Attachment>
    {
        /// <summary>
        /// This event is fired when the status of the <see cref="Add" /> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> AddProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Add"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler AddCompleted;
        /// <summary>
        /// This event fires, when the <see cref="Refresh"/> operation finishes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> RefreshProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Refresh"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler RefreshCompleted;
        /// <summary>
        /// This event is fired when the status of the <see cref="Remove" /> operation changes.
        /// </summary>
        public event ProgressChangedEventHandler<RequestStatus> RemoveProgressChanged;
        /// <summary>
        /// This event is fired when the <see cref="Remove"/> operation finishes.
        /// </summary>
        public event AsyncCompletedEventHandler RemoveCompleted;

        List<Attachment> _Attachments;

        internal AttachmentCollection(Uri url, Connection connection)
            : base(url, connection)
        {
            _Attachments = new List<Attachment>();
        }

        /// <summary>
        /// Returns the number of attachments on the current item.
        /// </summary>
        public int Count
        {
            get { return _Attachments.Count; }
        }

        /// <summary>
        /// Returns the <see cref="Attachment"/> with the specified index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Attachment this[int index]
        {
            get
            {
                return _Attachments[index];
            }
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to add an attachment to an item.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content for the attachment.</param>
        /// <returns></returns>
        public AsyncOperation<RequestStatus> GetAddOperation(string name, Stream stream)
        {
            Uri url;

            url = new Uri(Url, name);

            return new AsyncOperation<RequestStatus>(AddInternal(name, stream));
        }


        private IEnumerable<OperationAction> AddInternal(string name, Stream stream)
        {
            int index;
            Uri url;

            url = new Uri(Url.AbsoluteUri + "/" + name);

            using (AsyncOperation<RequestStatus> operation = Methods.Put(url, stream))
            {
                yield return new NestedOperation(operation, true);
            }

            index = -1;
            for (int i = 0; i < _Attachments.Count; i++)
            {
                if (!_Attachments[i].Name.Equals(name, StringComparison.CurrentCultureIgnoreCase)) continue;

                index = i;
                break;
            }
            if (index != -1) _Attachments[index] = new Attachment(Url, name, 0, Connection);
            else _Attachments.Add(new Attachment(Url, name, stream.CanSeek ? (int) stream.Length : 0, Connection));
        }

        /// <summary>
        /// Adds an attachment to the current item using the specified name and stream. The current thread is blocked until
        /// the operation completes.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content of the attachment.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Add(string name, Stream stream)
        {
            using (AsyncOperation<RequestStatus> operation = GetAddOperation(name, stream))
            {
                operation.Run();

                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Asynchronously adds an attachment to the current item using the specified name and stream.
        /// </summary>
        /// <param name="name">The name of the attachment</param>
        /// <param name="stream">The stream containing the content of the attachment.</param>
        public void AddAsync(string name, Stream stream)
        {
            AsyncOperation<RequestStatus> operation;

            AsyncOperationSupport.InitiatePendingAsyncOperation();

            operation = GetAddOperation(name, stream);

            operation.ProgressChanged += AddProgressChanged;
            operation.Completed += Add_Completed;
            operation.Error += Add_Error;

            AsyncOperationSupport.RegisterPendingAsyncOperation(operation);
            operation.Run();
        }

        void Add_Error(object sender, ExceptionEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnAddCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Add_Completed(object sender, CompletedEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnAddCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="AddCompleted"/> event.
        /// </summary>
        /// <param name="e">The parameters of the event.</param>
        protected virtual void OnAddCompleted(AsyncCompletedEventArgs e)
        {
            if (AddCompleted != null) AddCompleted(this, e);
        }

        /// <summary>
        /// Returns a <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to refresh this instance.
        /// </summary>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetRefreshOperation()
        {
            return new AsyncOperation<RequestStatus>(RefreshInternal());
        }

        private IEnumerable<OperationAction> RefreshInternal()
        {
            using (AsyncOperation<RequestStatus, PropertyContainer[]> operation = Methods.X_MS_EnumAtts(Url))
            {
                yield return new NestedOperation(operation, true);

                foreach (PropertyContainer container in operation.Result)
                {
                    _Attachments.Add(new Attachment(container, Connection));
                }
            }
        }

        /// <summary>
        /// Reads the attachment information from the current item. The current thread is blocked while the request is in progress.
        /// </summary>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Refresh()
        {
            using (AsyncOperation<RequestStatus> operation = GetRefreshOperation())
            {
                operation.Run();
                operation.WaitForCompletion();
            }
        }
        /// <summary>
        /// Asynchronously reads the attachment information from the current item. 
        /// </summary>
        public void RefreshAsync()
        {
            AsyncOperation<RequestStatus> operation;

            AsyncOperationSupport.InitiatePendingAsyncOperation();

            operation = GetRefreshOperation();
            AsyncOperationSupport.RegisterPendingAsyncOperation(operation);
            operation.ProgressChanged += RefreshProgressChanged;
            operation.Completed += Refresh_Completed;
            operation.Error += Refresh_Error;

            operation.Run();
        }

        void Refresh_Error(object sender, ExceptionEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnRefreshCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Refresh_Completed(object sender, CompletedEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnRefreshCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="RefreshCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters of the event.</param>
        protected virtual void OnRefreshCompleted(AsyncCompletedEventArgs e)
        {
            if (RefreshCompleted != null) RefreshCompleted(this, e);
        }

        /// <summary>
        /// Returns an <see cref="InfiniTec.Threading.AsyncOperation"/> which can be used to remoave the specified attachment.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        /// <returns>An <see cref="InfiniTec.Threading.AsyncOperation"/>.</returns>
        public AsyncOperation<RequestStatus> GetRemoveOperation(Attachment attachment)
        {
            return new AsyncOperation<RequestStatus>(RemoveInternal(attachment));
        }

        private IEnumerable<OperationAction> RemoveInternal(Attachment attachment)
        {
            if (!_Attachments.Contains(attachment)) throw new InvalidOperationException("The specified attachment is not an element of this AttachmentCollection instance");

            using (AsyncOperation<RequestStatus> operation = Methods.Delete(attachment.Url))
            {
                yield return new NestedOperation(operation, true);
            }
            _Attachments.Remove(attachment);
        }

        /// <summary>
        /// Removes the specified <see cref="Attachment"/> from the item. The current thread is blocked until the operation 
        /// completes.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void Remove(Attachment attachment)
        {
            using (AsyncOperation<RequestStatus> operation = GetRemoveOperation(attachment))
            {
                operation.Run();
                operation.WaitForCompletion();
            }
        }

        /// <summary>
        /// Removes the <see cref="Attachment"/> with the specified index. The current thread is blocked until the operation 
        /// completes.
        /// </summary>
        /// <param name="index">The index of the attachment to remove.</param>
        /// <exception cref="AsyncOperationFailedException">Thrown, when the operation fails. See <see cref="Exception.InnerException"/> for details.</exception>
        public void RemoveAt(int index)
        {
            Remove(this[index]);
        }
        
        /// <summary>
        /// Asynchronously removes the <see cref="Attachment"/> with the specified index from the item.
        /// </summary>
        /// <param name="index">The index of the <see cref="Attachment"/> to remove.</param>
        public void RemoveAtAsync(int index)
        {
            RemoveAsync(this[index]);
        }
        /// <summary>
        /// Asynchronously removes the specified <see cref="Attachment"/> from the item.
        /// </summary>
        /// <param name="attachment">The <see cref="Attachment"/> to remove.</param>
        public void RemoveAsync(Attachment attachment)
        {
            AsyncOperation<RequestStatus> operation;

            AsyncOperationSupport.InitiatePendingAsyncOperation();

            operation = GetRemoveOperation(attachment);

            operation.ProgressChanged += RemoveProgressChanged;
            operation.Completed += Remove_Completed;
            operation.Error += Remove_Error;

            AsyncOperationSupport.RegisterPendingAsyncOperation(operation);
            operation.Run();
        }

        void Remove_Error(object sender, ExceptionEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnRemoveCompleted(new AsyncCompletedEventArgs(e.Exception, false, null));
        }

        void Remove_Completed(object sender, CompletedEventArgs e)
        {
            AsyncOperationSupport.UnregisterPendingAsyncOperation();

            OnRemoveCompleted(new AsyncCompletedEventArgs(null, e.Status == OperationStatus.Aborted, null));
        }

        /// <summary>
        /// Executes the <see cref="RemoveCompleted"/> event.
        /// </summary>
        /// <param name="e">The event parameters.</param>
        protected virtual void OnRemoveCompleted(AsyncCompletedEventArgs e)
        {
            if (RemoveCompleted != null) RemoveCompleted(this, e);
        }

        #region IEnumerable<Attachment> Members

        /// <summary>
        /// Returns an <see cref="IEnumerator"/> which can be used to iterate through the collection of attachemnts.
        /// </summary>
        /// <returns>An <see cref="IEnumerator"/> instance.</returns>
        public IEnumerator<Attachment> GetEnumerator()
        {
            return _Attachments.GetEnumerator();
        }

        #endregion


        #region IEnumerable Members

        /// <summary>
        /// Returns an <see cref="IEnumerator"/> which can be used to iterate through the collection of attachemnts.
        /// </summary>
        /// <returns>An <see cref="IEnumerator"/> instance.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable)_Attachments).GetEnumerator();
        }

        #endregion
    }
}
